
class TemplateGen:

    def __init__(self):
      pass

    def generate_db(self):
      my_string = """
      [database]
      host = localhost
      port = 5432
      database = custom_rails_development
      username = "patcasrares"
      password = ""

      """

      with open("myfile.txt", "w") as f:
          f.write(my_string)



environ = {
    "ALLUSERSPROFILE": "C:\\ProgramData",
    "APPDATA": "C:\\Users\\rares\\AppData\\Roaming",
    "APPLICATION_INSIGHTS_NO_DIAGNOSTIC_CHANNEL": "true",
    "CHOCOLATEYINSTALL": "C:\\ProgramData\\chocolatey",
    "CHOCOLATEYLASTPATHUPDATE": "132163243699782294",
    "COMMONPROGRAMFILES": "C:\\Program Files\\Common Files",
    "COMMONPROGRAMFILES(X86)": "C:\\Program Files (x86)\\Common Files",
    "COMMONPROGRAMW6432": "C:\\Program Files\\Common Files",
    "COMPUTERNAME": "DESKTOP-2ML4R58",
    "COMSPEC": "C:\\WINDOWS\\system32\\cmd.exe",
    "DART_SDK": "C:\\Program Files\\Dart\\dart-sdk",
    "DRIVERDATA": "C:\\Windows\\System32\\Drivers\\DriverData",
    "FPS_BROWSER_APP_PROFILE_STRING": "Internet Explorer",
    "FPS_BROWSER_USER_PROFILE_STRING": "Default",
    "HOMEDRIVE": "C:",
    "HOMEPATH": "\\Users\\rares",
    "LOCALAPPDATA": "C:\\Users\\rares\\AppData\\Local",
    "LOGONSERVER": "\\\\DESKTOP-2ML4R58",
    "NUMBER_OF_PROCESSORS": "8",
    "ONEDRIVE": "C:\\Users\\rares\\OneDrive",
    "OS": "Windows_NT",
    "PATH": "C:\\WINDOWS\\system32;C:\\WINDOWS;C:\\WINDOWS\\System32\\Wbem;C:\\WINDOWS\\System32\\WindowsPowerShell\\v1.0\\;C:\\WINDOWS\\System32\\OpenSSH\\;C:\\Program Files\\Microsoft VS Code\\bin;C:\\Program Files\\Git\\cmd;C:\\Program Files (x86)\\ATI Technologies\\ATI.ACE\\Core-Static;C:\\Program Files\\nodejs\\;C:\\ProgramData\\chocolatey\\bin;C:\\WINDOWS\\system32\\config\\systemprofile\\.dnx\\bin;C:\\Program Files\\Microsoft DNX\\Dnvm\\;C:\\Program Files\\dotnet\\;C:\\Program Files\\Microsoft SQL Server\\130\\Tools\\Binn\\;C:\\Program Files\\Microsoft SQL Server\\Client SDK\\ODBC\\170\\Tools\\Binn\\;C:\\Program Fules;C:\\WINDOWS\\system32\\WindowsPowerShell\\v1.0\\Modules",
    "PUBLIC": "C:\\Users\\Public",
    "SESSIONNAME": "Console",
    "SYSTEMDRIVE": "C:",
    "SYSTEMROOT": "C:\\WINDOWS",
    "TEMP": "C:\\Users\\rares\\AppData\\Local\\Temp",
    "TMP": "C:\\Users\\rares\\AppData\\Local\\Temp",
    "USERDOMAIN": "DESKTOP-2ML4R58",
    "USERDOMAIN_ROAMINGPROFILE": "DESKTOP-2ML4R58",
    "USERNAME": "rares",
    "USERPROFILE": "C:\\Users\\rares",
    "VBOX_MSI_INSTALL_PATH": "C:\\Program Files\\Oracle\\VirtualBox\\",
    "VS140COMNTOOLS": "C:\\Program Files (x86)\\Microsoft Visual Studio 14.0\\Common7\\Tools\\",
    "WINDIR": "C:\\WINDOWS",
    "TERM_PROGRAM": "vscode",
    "TERM_PROGRAM_VERSION": "1.39.2",
    "LANG": "en_US.UTF-8",
    "COLORTERM": "truecolor",
    "SERVER_NAME": "DESKTOP-2ML4R58",
    "GATEWAY_INTERFACE": "CGI/1.1",
    "SERVER_PORT": "80",
    "REMOTE_HOST": "",
    "CONTENT_LENGTH": "",
    "SCRIPT_NAME": "",
    "SERVER_PROTOCOL": "HTTP/1.1",
    "SERVER_SOFTWARE": "WSGIServer/0.2",
    "REQUEST_METHOD": "GET",
    "PATH_INFO": "/",
    "QUERY_STRING": "",
    "REMOTE_ADDR": "127.0.0.1",
    "CONTENT_TYPE": "text/plain",
    "HTTP_HOST": "127.0.0.1",
    "HTTP_USER_AGENT": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:70.0) Gecko/20100101 Firefox/70.0",
    "HTTP_ACCEPT": "text/html,application/xhtml+xml,application/xml;q,0.9,*/*;q,0.8",
    "HTTP_ACCEPT_LANGUAGE": "en-US,en;q,0.5",
    "HTTP_ACCEPT_ENCODING": "gzip, deflate",
    "HTTP_CONNECTION": "keep-alive",
    "HTTP_UPGRADE_INSECURE_REQUESTS": "1",
    "HTTP_CACHE_CONTROL": "max-age,0"
}
def start_response(response_value, *args):
    return response_value, args



from custom_rails.request import Request
import json
from jinja2 import Template

# todo: Add more content types for various files
file_content_types = {
    'html': 'text/html',
    'htm':  'text/html',
    'css':  'text/css',
    'js':   'text/javascript',
    'mkv':  'video/mkv',
}

class Response:

    """
    Base Response Class
    """
    __slots__ = 'headers', 'status_code', 'start_response', 'content_type', 'response_content'

    def __init__(self,request: Request, status_code: str, content_type: str):
        self.headers = []
        self.status_code = status_code
        self.start_response = request.start_response
        self.content_type = content_type
        self.response_content = []
        
    def make_response(self):
        print(123)
        self.start_response(self.status_code, [('Content-Type', self.content_type)])
        return self.response_content         


class HttpResponse(Response):
    """
    Return pure http response when given a text as content
    """
    def __init__(self, request: Request, content, status_code='200 OK', content_type='text/html'):
        super().__init__( request, status_code, content_type)
        if type(content) == str:
            content = content.encode()
        self.response_content.append(content)

class JinjaResponse(Response):
    """
    Interpret erb file
    """
    def __init__(self, request: Request, filename, params={}, status_code='200 OK', content_type='text/html'):
        filename = "views/" + filename
        print(1)

        content = None
        try:
            with open(filename, 'rb') as f:
                content = f.read()
        except FileNotFoundError:
            print(f'File not found {filename}')
            raise Exception(f'could not find the file {filename}')
        print(2)
        content = content.decode('utf-8')
        print(content)
        template = Template(content)
        print(3)
        
        html = template.render(params)
        print(4)
        print(html)
        super().__init__( request, status_code, content_type)
        self.response_content.append(html.encode())


class RenderResponse(HttpResponse):
    """Uses HttpResponse to return http response given a template name and context dict"""
    # Todo: Implement a way to render dynamic content from the context into templates

    def __init__(self, request: Request, filename: str, context: dict = {}):
        try:
            with open(filename, 'r') as f:
                text = f.read()
        except FileNotFoundError:
            print(f'Error openning file {filename}')
            raise Exception(f'Could not find the file {filename}')
        super().__init__(request, text, '200 OK')


class JsonResponse(Response):
    """
    Return pure json response when given a text as content
    """
    def __init__(self, request: Request, content, status_code='200 OK', content_type='application/json'):
        content = json.dumps(content)
        super().__init__( request, status_code, content_type)
        self.response_content.append(content.encode())

class FileResponse(HttpResponse):
    def __init__(self,  request: Request, filename: str, file_root:str=""):
        try:
            with open(file_root+filename, 'rb') as f:
                content = f.read()
        except FileNotFoundError:
            print(f'File not found {filename}')
            raise Exception(f'could not find the file {filename}')
        
        content_type = file_content_types.get(filename.split('.')[-1], 'text/plain')
        
        super().__init__( request, content, '200 OK',content_type )

class ErrorResponse(Response):
    def __init__(self, request: Request, error_code: str):
        super().__init__(request, '404 Not Found', 'text/html')
        self.response_content.append("404 Not Found".encode())


class Http404(ErrorResponse):
    def __init__(self, request):
        super().__init__(request, '404 Not Found')




import psycopg2
import configparser

import re
import inflect


from custom_rails.db import *


class DbStatic:
    
    @classmethod
    def get_model(cls):

        name = cls.__name__
        # Remove "Model" from the end of the string
        name = re.sub("Model$", "", name)
        
        # Lowercase the string
        name = name.lower()
        
        # Pluralize the word
        p = inflect.engine()
        name = p.plural(name)
        return Db(name)
    
    @classmethod
    def execute_query(cls, query, params=None):
        model = cls.get_model()
        model.execute_query(query, params=None)

    @classmethod
    def execute_select(cls, query, params=None):
        model = cls.get_model()
        return model.execute_select(query, params=None)

    @classmethod
    def add(cls, data):
        model = cls.get_model()
        return model.add(data)

    @classmethod
    def get_all(cls):
        model = cls.get_model()
        return model.get_all()

    @classmethod
    def get_by_id(cls, id):
        model = cls.get_model()
        return model.get_by_id(id)

    @classmethod
    def update(cls, id, data):
        model = cls.get_model()
        model.update(id, data)

    @classmethod
    def delete(cls, id):
        model = cls.get_model()
        model.delete(id)

    @classmethod
    def add_column(cls, column_name, data_type):
        model = cls.get_model()
        model.add_column(column_name, data_type)

    @classmethod
    def delete_column(cls, column_name):
        model = cls.get_model()
        model.delete_column(column_name)




class Path:
    """
    Class to hold paths. makes it easier to check matches
    """

    __slots__ = 'path', 'func'

    def __init__(self, _path: str, _func):
        self.path = _path
        self.func = _func

    
    def match(self, _path):
        # function that is used to check whether a url route matches
        # that of the path.
        # Todo: Change to support dynamic urls like
        # /<int:id>/ just like django does so we can extract variables
        # from urls. 
        # Tip: You may need to learn regular expressions to match that
        if self.path == _path:
            return True
        return False
    

class Router:
    """
    Holds all routes in a list. When connection comes it is used to check which of those
    paths match the url
    """
    __slots__ = 'routes'
    def __init__(self, routes:list=None):
        self.routes:list = list(routes) if routes else []

    def add_route(self,  _path: Path):
        self.routes.append(_path)
        return True
        

    def get_route(self, path_):
        for path in self.routes:
            if path.match(path_): # we let the paths themselves check if they match url
                return path.func
        





"""
request.py
Holds the Request class that is basically our way of extracting all the things we need into 
a class representation
"""

import cgi


class PostBody:

    """
    Class to hold the all data sent during a request.
    eg. Data sent from forms are stored here
    """
    __slots__ = 'data'

    def __init__(self, data):
        self.data = data
        
    def get(self, key):
        value = self.data.get(key, [''])
        #if type(value) == str:
        #    value = cgi.escape(value) # we escape values to prevent script injection attacks
        return value
    
    def _set(self, key, value):
        self.data['key'] == value
    
    def __iter__(self):
        for key, value in self.data:
            yield key, value



        
class Request:
    """
    The base request object.
    """
    __slots__ = [
        "environ", 
        "http_host",
        "http_user_agent",
        "lang","method", 
        "path",
        "host_address", 
        "gateway_interface",
        "server_port", 
        "remote_host",
        "content_type",
        "content_length",
        "body",
        "query_string",
        "server_protocol",
        "server_software", 
        "start_response",
        "post",
        ]

    def __init__(self, environ, start_response):
        """The variables here just hold data extracted from the wsgi environ and the startresponse function"""
        self.environ = environ
        self.http_host = environ['HTTP_HOST']
        self.http_user_agent = environ['HTTP_USER_AGENT']
        self.lang = environ.get('LANG')
        self.method = environ.get('REQUEST_METHOD')
        self.path = environ.get('PATH_INFO')
        self.host_address = environ.get('HTTP_HOST')
        self.gateway_interface = environ.get('GATEWAY_INTERFACE')
        self.server_port = environ.get('SERVER_PORT')
        self.remote_host = environ.get('REMOTE_HOST')
        self.content_type = environ.get('CONTENT_TYPE')
        self.content_length = environ.get('CONTENT_LENGTH')
        self.body = environ.get('BODY')
        self.query_string = environ.get('QUERY_STRING')
        self.server_protocol = environ.get('SEVER_PROTOCOL')
        self.server_software = environ.get('SERVER_SOFTWARE')
        self.start_response = start_response
        self.post = PostBody({})
        self.parse_q()
        


    def parse_q(self):
        if self.method != "POST":
            return
        environ = self.environ        
        field_storage = cgi.FieldStorage(
            fp=environ['wsgi.input'],
            environ = environ,
            keep_blank_values= True
        )     
        for item in field_storage.list:
            print("Rares")
            print(item)
            if not item.filename:
                self.post.data[item.name] = item.value
            else:
                self.post.data[item.name] = item
                
    




import shutil
from pkg_resources import resource_filename

def copy_folder_contents(src_folder):
    template_dir = resource_filename("custom_rails", "templates")
    dest_dir = os.getcwd()
    shutil.copytree(template_dir, dest_dir, dirs_exist_ok=True)

def build_app():
    copy_folder_contents("./custom_rails/templates")


def make_app():
    print("okkkkkkk")
    build_app()

import os

cmd = "python3 main_app.py"

# Execute the command

def start_app():
    os.system(cmd)



from custom_rails.router import Router, Path

from unittest import TestCase

def func1(request):
    return 1

def func2(request):
    return 2

route_1 = Path('/', func1)
route_2 = Path('/2', func2)
routes = [route_1, route_2]

class TestRouter(TestCase):
    def test_create_router(self):
        router = Router()
        self.assertEqual(router.routes, [])
    def test_create_router_with_routes(self):
        
        router = Router(routes =routes)
        self.assertEqual(router.routes , routes)
        
    def test_router_set_route(self):
        router = Router()

        router.add_route(route_1)
        self.assertEqual(router.routes[0], route_1)

    

    def test_router_get_route(self):
        router = Router()
        router.add_route(route_1)
        self.assertEqual(router.get_route('/'), route_1.func)
    
        


from unittest import TestCase
from test.utils import environ, start_response
from custom_rails.request import Request
from custom_rails.response import Response, HttpResponse, Http404

class Testresponse(TestCase):
    def test_reponse_created(self):
        request = Request(environ, start_response)
        response = Response(request, '200 OK', 'text/html')
        self.assertEqual(response.headers, [])
        self.assertEqual(response.status_code, '200 OK')
        self.assertEqual(response.content_type ,'text/html' )
        self.assertEqual(response.response_content, [])
    
    def test_http_response(self):
        request = Request(environ, start_response)
        http_response = HttpResponse(request, 'Hello world')
        self.assertIsInstance(http_response.response_content, list)
        self.assertEqual(http_response.make_response(), [b'Hello world'])
    


import psycopg2

import configparser


class Db:
    def __init__(self, table_name):
        self.table_name = table_name
        config = configparser.ConfigParser()
        config.read('config.ini')

        database_config = config['database']
        host = database_config['host']
        port = database_config['port']
        database = database_config['database']
        username = database_config['username']
        password = database_config['password']

        print(f"Host: {host}")
        print(f"Port: {port}")
        print(f"Database: {database}")
        print(f"Username: {username}")
        print(f"Password: {password}")

        self.conn = psycopg2.connect(
            host=host,
            port=port,
            dbname=database
        )
        self.cursor = self.conn.cursor()
        self._create_table_if_not_exists()

    def _create_table_if_not_exists(self):
        query = f"""
            CREATE TABLE IF NOT EXISTS {self.table_name} (
                id SERIAL PRIMARY KEY,
                name TEXT NOT NULL,
                cnp TEXT NOT NULL,
                age INTEGER NOT NULL
            );
        """
        self.execute_query(query)

    def execute_query(self, query, params=None):
        if params is None:
            self.cursor.execute(query)
        else:
            self.cursor.execute(query, params)
        self.conn.commit()

    def execute_select(self, query, params=None):
        if params is None:
            self.cursor.execute(query)
        else:
            self.cursor.execute(query, params)
        rows = self.cursor.fetchall()
        return rows

    def add(self, data):
        columns = ', '.join(data.keys())
        placeholders = ', '.join('%s' for _ in data)
        query = f"INSERT INTO {self.table_name} ({columns}) VALUES ({placeholders})"
        params = tuple(data.values())
        self.execute_query(query, params)
        return self.cursor.lastrowid

    def get_all(self):
        query = f"SELECT * FROM {self.table_name}"
        rows = self.execute_select(query)
        items = []
        for row in rows:
            item = {}
            for i, column_name in enumerate(self.cursor.description):
                item[column_name.name] = row[i]
            items.append(item)
        return items

    def get_by_id(self, id):
        query = f"SELECT * FROM {self.table_name} WHERE id = %s"
        params = (id,)
        rows = self.execute_select(query, params)
        if len(rows) > 0:
            item = {}
            for i, column_name in enumerate(self.cursor.description):
                item[column_name.name] = rows[0][i]
            return item
        else:
            return None

    def update(self, id, data):
        set_columns = ', '.join(f"{column_name} = %s" for column_name in data.keys())
        query = f"UPDATE {self.table_name} SET {set_columns} WHERE id = %s"
        params = tuple(data.values()) + (id,)
        self.execute_query(query, params)

    def delete(self, id):
        query = f"DELETE FROM {self.table_name} WHERE id = %s"
        params = (id,)
        self.execute_query(query, params)

    def add_column(self, column_name, data_type):
        query = f"ALTER TABLE {self.table_name} ADD COLUMN {column_name} {data_type}"
        self.execute_query(query)

    def delete_column(self, column_name):
        query = f"ALTER TABLE {self.table_name} DROP COLUMN {column_name}"
        self.execute_query(query)

    def print_db_structure(self, path):
        # Establish a connection to the database
        conn = self.conn
        cursor = conn.cursor()

        # Get the table names in the database
        cursor.execute("SELECT table_name FROM information_schema.tables WHERE table_schema = 'public'")
        table_names = cursor.fetchall()

        with open(path, "w") as f:
            # Loop through each table and print its structure to the file
            for table_name in table_names:
                cursor.execute(f"SELECT column_name, data_type FROM information_schema.columns WHERE table_name = '{table_name[0]}'")
                columns = cursor.fetchall()
                f.write(f"Table: {table_name[0]}\n")
                for column in columns:
                    f.write(f"    Column: {column[0]}, Data Type: {column[1]}\n")
                f.write("\n")

        cursor.close()
        conn.close()




import argparse
from . import make_app, start_app

def main():
    parser = argparse.ArgumentParser(description="A command-line interface for custom_rails package")
    subparsers = parser.add_subparsers(title="commands", dest="command")

    new_parser = subparsers.add_parser("new", help="Create a new custom_rails app")
    new_parser.set_defaults(func=make_app)

    start_parser = subparsers.add_parser("start", help="Start the custom_rails app")
    start_parser.set_defaults(func=start_app)

    args = parser.parse_args()
    args.func()



from argparse import ArgumentParser

from custom_rails import make_app, start_app

if __name__ == "__main__":
    parser = ArgumentParser(
        prog="custom_rails",
        description="A command-line interface used for developing Rails-like web applications using Python and PyScript",
    )
    parser.add_argument("command", choices=["new", "start"])

    args = parser.parse_args()
    if args.command == "new":
        print("new")
        make_app()
        #copy_folder_contents("./custom_rails/templates")
    elif args.command == "start":
        print("start")
        start_app()


#### built in file do not 
import os
import inspect
import importlib.util
from custom_rails.router import Path
from wsgiref.simple_server import make_server
from custom_rails.response import HttpResponse, RenderResponse, JsonResponse, JinjaResponse

from custom_rails.db import Db

from custom_rails.start_rails_app import start_rails

start_rails()


from custom_rails.request import Request
from unittest import TestCase
from test.utils import start_response, environ


class Testrequest(TestCase):
    def test_create_request(self):
        request = Request(environ, start_response)
        self.assertEqual(request.http_host , environ['HTTP_HOST'])
        self.assertEqual(request.http_user_agent , environ['HTTP_USER_AGENT'])
        self.assertEqual(request.lang , environ.get('LANG'))
        self.assertEqual(request.method , environ.get('REQUEST_METHOD'))
        self.assertEqual(request.path , environ.get('PATH_INFO'))
        self.assertEqual(request.host_address , environ.get('HTTP_HOST'))
        self.assertEqual(request.gateway_interface , environ.get('GATEWAY_INTERFACE'))
        self.assertEqual(request.server_port , environ.get('SERVER_PORT'))
        self.assertEqual(request.remote_host , environ.get('REMOTE_HOST'))
        self.assertEqual(request.content_length , environ.get('CONTENT_LENGTH'))
        self.assertEqual(request.server_protocol , environ.get('SEVER_PROTOCOL'))
        self.assertEqual(request.server_software , environ.get('SERVER_SOFTWARE'))
        self.assertEqual(request.start_response , start_response)
        self.assertEqual(request.environ , environ)



"""
Holds the base class that puts all the pieces together
providing a symple interface for programming
"""

from custom_rails.router import Router, Path
from custom_rails.response import Response, FileResponse, Http404
from custom_rails.response import Request
import os

class App:
    def __init__(self):
        self.router = Router()
        self.static_dir = None  # holds the path where static files are stored
        self.static_path = None # specifies the url prefix for all incoming static file requests

    def set_static(self, static_path, static_dir):
        # function used to set static file related settings
        self.static_dir  = static_dir
        self.static_path = static_path
        
    
    def serve_static(self,request : Request):
        """A view to serve static files that is only added when the user sets the static variables"""
        new_path = request.path[len(self.static_path)::] # get the actual file path name 
        return FileResponse(request, os.path.join(self.static_dir, new_path))


    def set_routes(self, routes : list):
        # takes a list of Paths and add them to the app's router's list of routes
        for path in routes:
            self.router.add_route(path)

    def __call__(self, environ, start_response):
        # the wsgi callable
        try:
            request = Request(environ, start_response)
            # check if this request is directed at getting static files
            if self.static_path !=None and  request.path.startswith(self.static_path):
                response = self.serve_static(request)
                return response.make_response()
            else:
                func = self.router.get_route(request.path)
                if func is not None:
                    response: Response = func(request)
                    return response.make_response()  
                else:
                    print(f'route Not found : {request.path}')
        except Exception as e:
            print(e)

        response =  Http404(request)
        return response.make_response()


import os
import json
import inspect
import importlib.util
from custom_rails.router import Path
from wsgiref.simple_server import make_server
from custom_rails.response import HttpResponse, RenderResponse, JsonResponse, JinjaResponse

from custom_rails.db import Db

from custom_rails.app import App

from helpers import my_helper

import os
import glob
import importlib

# set the path to the folder containing the modules
module_path = "./db_models"

module_files = glob.glob(module_path + "/*.py")

# loop through the module files and import each module
for module_file in module_files:
    # remove the path and file extension to get the module name
    module_name = module_file.split("/")[-1][:-3]
    module_name = f"{module_path.replace('./', '')}.{module_name}"
    print(module_name)
    # import the module dynamically using importlib
    module = importlib.import_module(module_name)
    globals().update(vars(module))

module_path = "./helpers"

module_files = glob.glob(module_path + "/*.py")

# loop through the module files and import each module
for module_file in module_files:
    # remove the path and file extension to get the module name
    module_name = module_file.split("/")[-1][:-3]
    module_name = f"{module_path.replace('./', '')}.{module_name}"
    print(module_name)
    # import the module dynamically using importlib
    module = importlib.import_module(module_name)
    globals().update(vars(module))


def start_rails():
  # set the directory path
  Db("users").print_db_structure("schema.txt")
  dir_path = 'controllers'

  # get all the filenames in the directory
  filenames = os.listdir(dir_path)

  the_rsp = None


  routes = [
      #Path('/', print_received),
      #Path('/json/', json_point),
      #Path('/form/', form),
  ]
  cnt = 0
  # print the filenames
  for filename in filenames:
      if filename.endswith('.py'):
          # get the module name
          module_name = filename[:-3]
          # import os.path.join(dir_path, filename)
          path = os.path.join(dir_path, filename)

          content = None
          with open(path, 'rb') as f:
            content = f.read()
          content = content.decode('utf-8')
          
          content = content.split("def ")
          vf = 0
          for elem in content:
            if vf == 0:
              vf = 1
              continue

            cnt = cnt + 1
            name = elem.split("(")[0]
            fct = "def " + elem
            the_rsp = None
            the_request = None
            fct =  fct + "\nprint(request)\nthe_rsp = " + name + "(request)\nprint(the_rsp)"
            print(fct)
            current_fct = fct
            def aux_fct(request, fct = current_fct): 
              print(fct)
              the_rsp = None
              the_locals = {'request': request}
              exec(fct, globals(), the_locals )
              the_rsp = the_locals.get('the_rsp')
              return the_rsp
            print("/" + module_name + "/" + name)
            routes = routes + [Path("/" + module_name + "/" + name, aux_fct)]

  def default_path(request):
    return HttpResponse(request,"""
    <html>
    <body>
    Welcome!!!
    </body>
    </html>
    """)
  routes = routes + [Path("/", default_path)]

  print(routes)

  route_names = []
  for route in routes:
      route_names.append(route.path)

  with open('routes.txt', 'w') as f:
      f.write('\n'.join(route_names))

  app = App()
  app.set_static('/static/', '.')

  app.set_routes(routes)

  server = make_server('127.0.0.1', 3000, app)
  server.serve_forever()



from setuptools import setup

setup(
    name="custom_rails",
    keywords=["wsgi", "framework", ],
    version="0.0.6",
    maintainer="Patcas Rares",
    package_data={"custom_rails": ["templates/*"]},
    description="An educational wsgi based python web framework",
    packages=["custom_rails"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    entry_points={
        'console_scripts': [
            'custom_rails = custom_rails.cli:main',
        ]
    }
)



from unittest import TestCase
from custom_rails.app import App
from custom_rails.router import Router

from custom_rails.response import Response, Http404
from test.utils import environ, start_response

def static_func(*args):
    return 'called'
class Testapp(TestCase):
    def test_create_app(self):
        app = App()
        self.assertEqual(app.static_dir, None)
        self.assertEqual(app.static_path, None)
        self.assertIsInstance(app.router, Router)
        self.assertEqual(len(app.router.routes), 0)

    def test_set_static(self):
        app = App()
        app.set_static('/static','static_folder')
        self.assertEqual(app.static_dir, 'static_folder')
        self.assertEqual(app.static_path, '/static')
        self.assertEqual(len(app.router.routes), 0)
    
    def test_called(self):
        env = environ.copy()
        env['PATH_INFO'] = '/'
        app = App()
        self.assertIsInstance(app(environ, start_response), list)
    def test_called_with_non_existent_route(self):
        env = environ.copy()
        env['PATH_INFO'] = '/does/not/exist/'
        app = App()
        self.assertEqual(app(environ, start_response), [b'404 Not Found'])
        